"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const path_1 = __importDefault(require("path"));
const mri_1 = __importDefault(require("mri"));
const Command_1 = __importStar(require("./Command"));
const utils_1 = require("./utils");
class CAC extends events_1.EventEmitter {
    constructor() {
        super();
        this.commands = [];
        this.globalCommand = new Command_1.GlobalCommand(this);
        this.globalCommand.usage('<command> [options]');
    }
    /**
     * Add a global usage text.
     *
     * This is not used by sub-commands.
     */
    usage(text) {
        this.globalCommand.usage(text);
        return this;
    }
    /**
     * Add a sub-command
     */
    command(rawName, description, config) {
        const command = new Command_1.default(rawName, description, config, this);
        command.globalCommand = this.globalCommand;
        this.commands.push(command);
        return command;
    }
    /**
     * Add a global CLI option.
     *
     * Which is also applied to sub-commands.
     */
    option(rawName, description, config) {
        this.globalCommand.option(rawName, description, config);
        return this;
    }
    /**
     * Show help message when `-h, --help` flags appear.
     *
     */
    help(callback) {
        this.globalCommand.option('-h, --help', 'Display this message');
        this.globalCommand.helpCallback = callback;
        return this;
    }
    /**
     * Show version number when `-v, --version` flags appear.
     *
     */
    version(version, customFlags = '-v, --version') {
        this.globalCommand.version(version, customFlags);
        return this;
    }
    /**
     * Add a global example.
     *
     * This example added here will not be used by sub-commands.
     */
    example(example) {
        this.globalCommand.example(example);
        return this;
    }
    /**
     * Output the global help message
     *
     * This will also call `process.exit(0)` to quit the process.
     */
    outputHelp() {
        this.globalCommand.outputHelp();
    }
    /**
     * Output the version number.
     *
     * This will also call `process.exit(0)` to quit the process.
     */
    outputVersion() {
        this.globalCommand.outputVersion();
    }
    setParsedInfo({ args, options, rawOptions }, matchedCommand) {
        this.args = args;
        this.options = options;
        this.rawOptions = rawOptions;
        if (matchedCommand) {
            this.matchedCommand = matchedCommand;
        }
        return this;
    }
    /**
     * Parse argv and run command action if found.
     */
    parse(argv = process.argv) {
        this.rawArgs = argv;
        this.bin = argv[1] ? path_1.default.basename(argv[1]) : 'cli';
        let shouldParse = true;
        // Search sub-commands
        for (const command of this.commands) {
            const mriOptions = utils_1.getMriOptions(this.globalCommand, command);
            const mriResult = this.mri(argv.slice(2), mriOptions);
            const commandName = mriResult.args[0];
            if (command.isMatched(commandName)) {
                shouldParse = false;
                const parsedInfo = Object.assign({}, mriResult, { args: mriResult.args.slice(1) });
                this.setParsedInfo(parsedInfo, command);
                this.emit(`command:${commandName}`, command);
            }
        }
        if (shouldParse) {
            // Search the default command
            for (const command of this.commands) {
                if (command.name === '') {
                    shouldParse = false;
                    const mriOptions = utils_1.getMriOptions(this.globalCommand, command);
                    const mriResult = this.mri(argv.slice(2), mriOptions);
                    this.setParsedInfo(mriResult, command);
                    this.emit(`command:!`, command);
                }
            }
        }
        if (shouldParse) {
            const globalMriOptions = utils_1.getMriOptions(this.globalCommand);
            const mriResult = this.mri(argv.slice(2), globalMriOptions);
            this.setParsedInfo(mriResult);
        }
        if (this.options.help && this.globalCommand.hasOption('help')) {
            this.outputHelp();
        }
        if (this.options.version &&
            this.globalCommand.hasOption('version') &&
            this.globalCommand.versionNumber) {
            this.outputVersion();
        }
        const parsedArgv = { args: this.args, options: this.options };
        if (this.matchedCommand) {
            this.runCommandAction(this.matchedCommand, parsedArgv);
        }
        else {
            this.emit('command:*');
        }
        return parsedArgv;
    }
    mri(argv, mriOptions) {
        let argsAfterDoubleDashes = [];
        const doubleDashesIndex = argv.indexOf('--');
        if (doubleDashesIndex > -1) {
            argsAfterDoubleDashes = argv.slice(0, doubleDashesIndex);
            argv = argv.slice(doubleDashesIndex + 1);
        }
        const parsed = mri_1.default(argv, mriOptions);
        const args = parsed._;
        delete parsed._;
        const options = {
            '--': argsAfterDoubleDashes
        };
        for (const key of Object.keys(parsed)) {
            const keys = key.split('.').map((v, i) => {
                return i === 0 ? utils_1.camelcase(v) : v;
            });
            utils_1.setDotProp(options, keys, parsed[key]);
        }
        return {
            args,
            options,
            rawOptions: parsed
        };
    }
    runCommandAction(command, { args, options }) {
        if (!command.commandAction)
            return;
        command.checkUnknownOptions();
        command.checkRequiredOptions();
        const minimalArgsCount = command.args.filter(arg => arg.required).length;
        if (args.length < minimalArgsCount) {
            console.error(`error: missing required args for command \`${command.rawName}\``);
            process.exit(1);
        }
        const actionArgs = [];
        command.args.forEach((arg, index) => {
            if (arg.variadic) {
                actionArgs.push(args.slice(index));
            }
            else {
                actionArgs.push(args[index]);
            }
        });
        actionArgs.push(options);
        return command.commandAction.apply(this, actionArgs);
    }
}
exports.default = CAC;
